@@ -1,7 +1,7 @@
 #' Estimate Kernel Isotope Niche
 #'
 #' Calculates the 2D kernel for isotopic values at multiple confidence levels. Returns a list of
-#' SpatialPolygonsDataFrame, each list item representing the grouping variable (i.e. species).
+#' sf data frames, each list item representing the grouping variable (i.e. species).
 #'
 #' @param data data.frame object containing columns of isotopic values and grouping variables
 #' @param x character giving the column name of the x coordinates
@@ -28,10 +28,11 @@
 #' * ref - Uses MASS::bandwidth.nrd for both x and y separately, dividing values by 4 to match the scale of ks methods. Values can be obtained using bw_ref(). See MASS:kde2d() for details (i.e. the function divides the values by 4).
 #'
 #'
-#' @return A class rKIN object containing a list of SpatialPolygonsDataFrame, each list item representing the grouping variable.
+#' @return A class rKIN object containing a list of sf data frames, each list item representing the grouping variable.
 #' @author Shannon E. Albeke, Wyoming Geographic Information Science Center, University of Wyoming
 #' @export
 #' @import ks
+#' @import sf
 #' @examples
 #' library(rKIN)
 #' data("rodents")
@@ -83,9 +84,10 @@ estKIN <- function(data, x, y, h = "ref", hval = NULL, group, levels = c(50, 75,
   # Loop through each unique value of the group column
   grp<- unique(as.character(data[, group]))
   #create the output object for SpatialPolygonsDataFrame(s)
-  spdf.list<- list()
+  #spdf.list<- list()
   # create the output object for SpatialPointsDataFrame(s)
-  spts.list<- list()
+  #spts.list<- list()
+  sf.tmp<- createSPDF()
   for(g in 1:length(grp)){
     # Test for the number of samples. If too small, kick an error
     if(nrow(data[data[, group] == grp[g] , ]) < 10 & smallSamp == FALSE)
@@ -139,27 +141,34 @@ estKIN <- function(data, x, y, h = "ref", hval = NULL, group, levels = c(50, 75,
     rq<- getKernelThreshold(kde$z, levels)
 
     df.g<- data[data[, group] == grp[g] , ]
-    # create the spatial points data.frame
-    # populate the points into the spdf
-    spts.tmp<- sp::SpatialPointsDataFrame(coords = df.g[ , c(x, y)],
-                                          data = data.frame(Method = rep("Kernel", nrow(df.g)),
-                                                            Group = rep(grp[g], nrow(df.g)),
-                                                            x = df.g[, x], y = df.g[, y]))
+
+
+    df.tmp <- data.frame(Method = rep("Kernel", nrow(df.g)),
+                         Group = rep(grp[g], nrow(df.g)),
+                         x = df.g[, x], y = df.g[, y])
+
+    if (!exists("sfpts.tmp")) {
+      sfpts.tmp <- sf::st_as_sf(df.tmp, coords = c("x", "y"), remove = FALSE)
+      names(sfpts.tmp)[3:4] <- c(x, y)
+    }
+    else {
+      temp <- sf::st_as_sf(df.tmp, coords = c("x", "y"), remove = FALSE)
+      names(temp)[3:4] <- c(x, y)
+      sfpts.tmp <- rbind(sfpts.tmp, temp)
+    }
     # set column names to the input values
-    names(spts.tmp)[3:4]<- c(x, y)
-    # Convert 2D kernel theshold values to contourLines for each level
-    # create list for spatial polygons
-    sp.tmp<- createSPDF() # use helper function
+
     for(lev in 1:length(levels)){
       cL <- grDevices::contourLines(x = kde$x, y = kde$y, z = kde$z, levels = rq$Threshold[lev])
 
       # Function was directly copied from raster package
-      .contourLines2LineList <- function(cL) {
+      .contourLines2LineList_sf <- function(cL) {
         n <- length(cL)
         res <- vector(mode = "list", length = n)
         for (i in 1:n) {
           crds <- cbind(cL[[i]][[2]], cL[[i]][[3]])
-          res[[i]] <- sp::Line(coords = crds)
+
+          res[[i]] <- sf::st_linestring(x = crds)
         }
         res
       }
@@ -168,96 +177,63 @@ estKIN <- function(data, x, y, h = "ref", hval = NULL, group, levels = c(50, 75,
         stop("no contour lines")
       cLstack <- tapply(1:length(cL), sapply(cL, function(x) x[[1]]),
                         function(x) x, simplify = FALSE)
-      df <- data.frame(ConfInt = levels[lev])
+
+      #df <- data.frame(ConfInt = levels[lev])
+
       m <- length(cLstack)
+
       res <- vector(mode = "list", length = m)
-      IDs <- paste("C", 1:m, sep = "_")
-      row.names(df) <- IDs
-      for (i in 1:m) {
-        res[[i]] <- sp::Lines(.contourLines2LineList(cL[cLstack[[i]]]), ID = IDs[i])
+      #IDs <- paste("C", 1:m, sep = "_")
+      #row.names(df) <- IDs
+
+      res <- sf::st_sfc(.contourLines2LineList_sf(cL))
+
+      res <- sf::st_cast(res, to = "POLYGON")
+
+      # vertices <- matrix(c(0, 0, 4, 2, 3, 4, 0, 0), ncol = 2, byrow = TRUE)
+      # triangle <- sf::st_polygon(list(vertices))
+      # triangle_sf <- sf::st_sfc(triangle)
+      #
+      # vertices2 <- matrix(c(1.5, 1.5, 2, 2.5, 3, 2.5, 1.5, 1.5), ncol = 2, byrow = TRUE)
+      # triangle2 <- sf::st_polygon(list(vertices2))
+      # triangle_sf2 <- sf::st_sfc(triangle2)
+      #
+      # innerVertices <- matrix(c(-26.5, 4, -24, 4.5, -23.5, 5.5, -26.5, 4), ncol = 2, byrow = TRUE)
+      # innerTriangle <- sf::st_polygon(list(innerVertices))
+      # innerTriangle_sf <- sf::st_sfc(innerTriangle)
+      # plot(innerTriangle_sf)
+
+      #res <- c(res, triangle_sf, innerTriangle_sf, triangle_sf2)
+
+      if (length(res) > 1) {
+        # find all the outer polygons containing other polygons
+        testContains <- sf::st_contains_properly(res)
+
+        # get the index of these outer polygons
+        nonemptyIndex <- which(lengths(testContains) > 0)
+        if (length(nonemptyIndex) > 0) {
+
+          # erase represents the inner polygon we want to remove
+          erase <- sf::st_union(res[unlist(testContains[nonemptyIndex])])
+          # erased should be the outer polygon without the inners
+          res <- sf::st_difference(res, erase)
+        }
       }
-      SL <- sp::SpatialLines(res)
-      SL<- sp::SpatialLinesDataFrame(SL, data = df)
-      # end Contour from raster function
 
-      #////////////////////////////////////////////////
-      # convert the Lines to a set of polygons
-      cl.out<- list()
-      for(cl in 1:length(SL@lines[[1]]@Lines)){
-        # first test if the ring will close
-        a<- SL@lines[[1]]@Lines[[cl]]@coords
-        if(sp::spDistsN1(matrix(a[1,], ncol = 2),matrix(a[nrow(a),], ncol = 2)) > 0){
-          # create a single polygon
-          rstdy<- try(sp::Polygon(SL@lines[[1]]@Lines[[cl]]), silent = TRUE)
-          if(!inherits(rstdy, "try-error")){
-            #create a list of polygons
-            cl.out<- c(cl.out, sp::Polygons(list(rstdy), ID = cl))
-          }
-        }
-        else{
-          #create a single polygon
-          rstdy<- sp::Polygon(SL@lines[[1]]@Lines[[cl]])
-          #create a list of polygons
-          cl.out<- c(cl.out, sp::Polygons(list(rstdy), ID = cl))
-        }
-      }#close cl loop
+      ## Union erased together to smash into sf object that will be sent for CI and group
+      outerPolys <- sf::st_union(res)
 
-      # now make it spatially aware
-      if(length(cl.out)>0){
-        pcont<- sp::SpatialPolygons(cl.out)
-        # now iteratively test for holes within the polygons
-        repeat{
-          # check for something to have a hole
-          hole<- data.frame(rgeos::gContainsProperly(pcont, byid = TRUE))
-          # now loop through each column to determine which poly contains another
-          for(rw in 1:ncol(hole)){
-            is.hole<- character()
-            is.hole<- rownames(hole[which(hole[,rw]==TRUE), ])
-            # only 1 row, do this one time
-            if(length(is.hole)==1){
-              a.hole<- which(row.names(pcont)==is.hole)
-              # pass to the function
-              pcont<- makeHole(pcont, rw, a.hole)
-            }# close if
-            # now test if we need to revist the same column
-            else{
-              if(length(is.hole)>1){
-                # only work on the first of a vector
-                a.hole<- which(row.names(pcont)==is.hole[1])
-                # pass to the function
-                pcont<- makeHole(pcont, rw, a.hole)
-                # OK, just overwrote the polygon with the hole get out of loop
-                break
-              }# close hole >1
+      sfObj <- sf::st_as_sf(cbind(data.frame(Method = "Kernel", Group = grp[g], ConfInt = levels[lev], ShapeArea = NA_real_), outerPolys))
+      # Not accessing the specific geometry column is this okay?
+      sfObj$ShapeArea <- sf::st_area(outerPolys)
 
-            }# close else
-          }# close rw
-          # need to get out of the repeat
-          if(length(which(apply(hole, 2, any)==TRUE)) < 1) break
-        }# close repeat
+      sf.tmp <- rbind(sf.tmp, sfObj)
 
-        # now merge the remaining polys into a multipart
-        noply<- rgeos::gUnaryUnion(pcont)
-        noply<- sp::SpatialPolygonsDataFrame(noply, data = data.frame(Method = "Kernel", Group = grp[g], ConfInt = levels[lev], ShapeArea = noply@polygons[[1]]@area), match.ID = FALSE)
-        noply<- sp::spChFIDs(noply, x = as.character(lev))
-        #////////////////////////////////////////////////
-
-        sp.tmp<- sp::rbind.SpatialPolygonsDataFrame(sp.tmp, noply)
-      }# close cl.out if
     }# end levels loop
-    # add the group polygon to the list of outputs
-    spdf.list<- c(spdf.list, sp.tmp)
-    # add the group points to the list of outputs
-    spts.list<- c(spts.list, spts.tmp)
+
   }# close group loop
   # describe the polygons
-  names(spdf.list)<- grp
-  class(spdf.list)<- "estObj"
-  # describe the points
-  names(spts.list)<- grp
-  class(spts.list)<- "estInput"
-  # combine the polygons and points
-  kud<- list(estInput = spts.list, estObj = spdf.list)
+  kud<- list(estInput = sfpts.tmp, estObj = sf.tmp)
   attr(kud, "package")<- "rKIN"
   return(kud)
 } # close function
